# encoding: utf-8

from __future__ import unicode_literals

from abc import ABCMeta
from abc import abstractmethod

import six

from data_import.logging import ILogable


class ILoader(object):
    """Интерфейс загрузчика."""

    __metaclass__ = ABCMeta

    @abstractmethod
    def load(self, rows):
        """Выполняет загрузку строк."""


class AbstractLoader(ILoader, ILogable):
    """Абстрактный загрузчик строк."""

    def __init__(self, logger, params=None):
        self.logger = logger
        self.params = params

    def _pre_load(self, row):
        """Действия, выполняемые перед загрузкой.

        Предназначен для переопределения в классах-наследниках для выполнения
        нестандартных действий.
        """

    def _post_load(self, result, row):
        """Действия, выполняемые после загрузки.

        Предназначен для переопределения в классах-наследниках для выполнения
        нестандартных действий.
        """

    @abstractmethod
    def _load(self, row):
        """Метод загрузки одной строки."""

    def load(self, rows):
        for row in rows:
            try:
                self._pre_load(row)
                result = self._load(row)
                self._post_load(result, row)
            except Exception as exception:  # pylint: disable=W0703
                self.logger.log_from_exception(
                    exception, prefix=six.text_type(row)
                )
