# encoding: utf-8
from __future__ import unicode_literals

from os import path
from os import walk
from shutil import rmtree

import six

from ... import configuration
from ... import ImportException


def collect(destination):
    """Сбор распакованных файлов.

    :param basestring destination: путь к директории распаковки
    """
    for _, _, files in walk(destination):
        for filename in files:
            yield path.join(destination, filename)


def prepare_dir(source):
    """Подготовка директории для распаковки.

    :param basestring source: путь к папке, где производится распаковка архива
    """
    destination = path.join(path.dirname(source), 'tmp')

    # Очистка, на случай совпадения с предыдущим запуском
    if path.exists(destination):
        rmtree(destination)

    return source, destination


def detect_config(file_path):
    """Определение конфига по файлу.

    Возвращается первый конфиг, правила которого совпадают с
    колонками файла.

    :param basestring file_path: путь к файлу
    """

    file_ext = path.splitext(file_path)[1].strip('.')
    ext_configs = configuration.registry.get_configs(file_ext)

    for priority, config in enumerate(reversed(ext_configs)):
        rules = config.rules()
        config_headers = (
            rule['header'] for rule in six.itervalues(rules)
        )

        logger = config.logger()
        try:
            file_headers = config.parser(file_path, rules, logger).headers
        except ImportException:
            file_headers = tuple()

        if sorted(config_headers) == sorted(file_headers):
            return config, priority


def files_by_priority(destination):
    """Определение приоритета загрузки файлов при массовом импорте.

    :param basestring destination: путь к папке с файлами
    """
    result = []
    for file_path in collect(destination):
        detected = detect_config(file_path)
        if not detected:
            continue

        config, priority = detected
        result.append((priority, config, file_path))
    return reversed(sorted(result))
