# encoding: utf-8

from __future__ import unicode_literals

from abc import ABCMeta
from abc import abstractmethod
from abc import abstractproperty

import six


class ILogable(object):
    """Интерфейс логируемого объекта."""

    __metaclass__ = ABCMeta

    logger = None


class ILogger(object):
    """Интерфейс логирующего объекта."""

    __metaclass__ = ABCMeta

    @abstractproperty
    def messages(self):
        """Возвращет коллекцию сообщений ."""

    @abstractmethod
    def log(self, message, prefix=None):
        """Сохраняет переданное сообщение."""


class Logger(ILogger):
    """Класс, логирующий процесс импорта."""

    def __init__(self):
        self._messages = []

    @property
    def messages(self):
        """Список накопленных сообщений."""
        return self._messages

    def log(self, message, prefix=None):
        """Сохраняет переданное сообщение."""

        if isinstance(message, Exception):
            self.log_from_exception(message, prefix)
        else:
            if prefix:
                message = '{0} - {1}'.format(prefix, message)

            self._messages.append(message)

    def log_from_exception(self, exception, prefix=None):
        """Логирует сообщение путем извлечения его из исключения."""

        message = six.text_type(exception)
        self.log(message, prefix)
